#!/bin/bash
# Script to automate the installation of the backup stuff.
# Paul S. Crawford psc@sat.dundee.ac.uk
#
# Released under GPL v3
#
# psc 06 Oct 2009 - Clarified licence as GPL v3
# psc 21 Jan 2011 - Added --preserve=timestamps to copy commands.
# psc 31 Jan 2011 - Now comes with "home.exclu--" file, etc, and copied to "home.exclude"
#					as required. Saved pasting from updated archive clobbering your own files!
# psc 12 Nov 2013 - Now adds cron job for systems left running long-term.
# psc 19 May 2014 - Make sure cron job permissions are correct. Add logrotate installation.

# Some good scripting checks.
set +o errexit
set +o nounset

function print_usage () {
	printf "Usage: %s [options]\n" $( basename $0) >&2
	echo "Options are from the following:" >&2
	echo "-I 0|1   0 = Don't install any files (leave unchanged)." >&2
	echo "         1 = Install the automated backup (default), this" >&2
	echo "             will backup existing configuration files." >&2
	echo "" >&2
	echo "-R 0|1|2     Controls automated back-up on reboot with" >&2
	echo "         0 = Don't call on reboot or shut-down" >&2
	echo "         1 = Call on shut-down (default)." >&2
	echo "         2 = Call on reboot & shut-down." >&2
	echo "" >&2
	echo "-X      Remove the script completely, delete ALL installed!" >&2

return 0
}

# Must be root to do this?

if [ ! `id -u` = 0 ]; then
	echo "Sorry, you have to be 'root' (or use 'sudo') to"
	echo "have any chance of running this set-up script."
	echo ""
	echo "Did you read the README file or check to see what"
	echo "this script did before running it?"
	echo ""
	print_usage
	exit 1
fi

# Parse the command line for suitable options. Start by clearing everything set here.

OPT_LIST='I:R:X'

doinstall=1
dorestart=1
doremove=0

# Parse command line 1st time for lock-override, etc.
while getopts $OPT_LIST OPTION
	do
		case $OPTION in
		X)	doremove=1
			doinstall=0
			;;

		I)	doinstall="$OPTARG"
			;;

		R)	dorestart="$OPTARG"
			;;

		?)	print_usage
			exit 1
			;;
	  esac
	done

# Check for exisitng files, of not copy example 'home' case, etc.
# First copy over all something.inclu-- and something.exclu-- files:

	for fname in *.[ie]*--
		do
		dest=$(echo "$fname" | sed 's/--/de/g')
		if [ ! -e "$dest" ] ; then
			echo "Creating default '$dest' file"
			cp -p "$fname" "$dest"
		fi
		done

# Then copy over all "*.co--" file(s) (probably just pbackupd.conf just now).
	for fname in *.co--
		do
		dest=$(echo "$fname" | sed 's/--/nf/g')
		if [ ! -e "$dest" ] ; then
			echo "Creating default '$dest' file"
			cp -p "$fname" "$dest"
		fi
		done

# The configuration stuff first.

CFG_DEST=/etc/pbackup

if [ $doremove -eq 1 ] && [ -d "$CFG_DEST" ] ; then
	# Remove ALL old stuff.
	echo "Removing all of $CFG_DEST"
	rm -rf $CFG_DEST
fi

if [ $doinstall -eq 1 ] ; then
	# First create the configuration directory and copy the
	# configuration file to it.
	if [ ! -d "$CFG_DEST" ] ; then
		echo "Creating $CFG_DEST"
		mkdir $CFG_DEST
	else
		echo "Archiving current configuration files"
		pushd $CFG_DEST > /dev/null
		tar -czf archive-$(date +%Y%m%d-%H.%M.%S).tgz --exclude='*.tgz' *
		popd  > /dev/null
	fi

	echo "Copying configuration files to $CFG_DEST"

	cp --preserve=timestamps *.include *.exclude *.conf $CFG_DEST

	# Now change the settings so only root can modify things.
	# This is VERY important for system security!

	chown root:root $CFG_DEST
	chmod 755 $CFG_DEST
	chown root:root $CFG_DEST/*
	chmod 644 $CFG_DEST/*
fi


# Same for the actual backup/restore scripts. They have to be in
# the path that is present in single user mode.

USR_SCRIPTS='run_pbackup run_prestore'
SCRIPT_DEST=/usr/sbin

for prog in $(echo "$USR_SCRIPTS" )
	do
	if [ $doremove -eq 1 ] && [ -e "$SCRIPT_DEST/$prog" ] ; then
		echo "Removing $SCRIPT_DEST/$prog"
		rm -f "$SCRIPT_DEST/$prog"
	fi

	if [ $doinstall -eq 1 ] ; then
		echo "Copying $prog to $SCRIPT_DEST"
		cp --preserve=timestamps $prog $SCRIPT_DEST
		chown root:root $SCRIPT_DEST/$prog
		chmod 755 $SCRIPT_DEST/$prog
	fi
	done

# Now for the automatic bit at shut down.

DAEMON='pbackupd'
RUNDIR='/etc/init.d'

if [ $doremove -eq 1 ] && [ -e "$RUNDIR/$DAEMON" ] ; then
	echo "Removing $RUNDIR/$DAEMON"
	rm -f "$RUNDIR/$DAEMON"
fi

if [ $doinstall -eq 1 ] ; then
	echo "Installing $DAEMON to $RUNDIR"
	cp --preserve=timestamps $DAEMON $RUNDIR
	chown root:root $RUNDIR/$DAEMON
	chmod 755 $RUNDIR/$DAEMON
fi

# Remove current settings (if any)
update-rc.d -f $DAEMON remove

# Set to run on shut down (level 0 = system halt). Note we have 'stop' actually doing the backup!
# Also restart? (run level 6) Can be tedious after update.

case $dorestart in
	0)	echo "Removed any previous automatic call"
	;;
	1) 	echo "Configuring for automatic backup on shut-down..."
#		[ -x "$RUNDIR/$DAEMON" ] && update-rc.d $DAEMON start 00 0 .
		[ -x "$RUNDIR/$DAEMON" ] && update-rc.d $DAEMON stop 00 0 .
	;;
	2)	echo "Configuring for automatic backup on both restart and shut-down..."
#		[ -x "$RUNDIR/$DAEMON" ] && update-rc.d $DAEMON start 00 0 6 .
		[ -x "$RUNDIR/$DAEMON" ] && update-rc.d $DAEMON stop 00 0 6 .
	;;
	*)	echo "Unknown option \"$dorestart\" for automatic call settings!"
	;;
esac

# Set up cron to run backup in the middle of the night.

psrc=pbackup.cron
pcron=pbackup
crond=/etc/cron.d

if [ $doremove -eq 0 ] ; then
	if [ -f "$psrc" ] && [ -d "$crond" ] ; then
		echo "Setting up periodic backup using $crond/$pcron"
		cp "$psrc" "$crond/$pcron"
		chown root:root "$crond/$pcron"
		chmod 644 "$crond/$pcron"
		service cron restart
	fi
else
	if [ -f "$crond/$pcron" ] ; then
		echo "Removing $crond/$pcron"
		rm -f "$crond/$pcron"
		service cron restart
	fi
fi

# Set up log file rotation.

psrc=pbackup.logrotate
plog=pbackup
logd=/etc/logrotate.d

if [ $doremove -eq 0 ] ; then
	if [ -f "$psrc" ] && [ -d "$logd" ] ; then
		echo "Setting up periodic backup using $logd/$plog"
		cp "$psrc" "$logd/$plog"
		chown root:root "$logd/$plog"
		chmod 644 "$logd/$plog"
	fi
else
	if [ -f "$logd/$plog" ] ; then
		echo "Removing $logd/$plog"
		rm -f "$logd/$plog"
	fi
fi

exit 0

# End of script.
